@@ -145,8 +145,7 @@ storage = os.path.join(datahome, "storage")
 
 resfiles = [
 	# Only essentials
-	"argyll_instruments.json",
-	"lang/en.json",
+	"lang/en.yaml",
 	"beep.wav",
 	"camera_shutter.wav",
 	"linear.cal",
@@ -367,16 +366,21 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 						img.Rescale(w, h, quality=quality)
 				factors = None
 				if (not inverted and len(parts) > 2 and parts[-3] == "icons" and
+					(ow, oh) != (10, 10) and oname not in ("black_luminance",
+														   "check_all",
+														   "contrast",
+														   "luminance") and
 					max(wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW)[:3]) < 102):
+					# Automatically invert B&W image if background is dark
+					# (exceptions do apply)
 					if not img:
 						img = bmp.ConvertToImage()
 					if img.IsBW():
 						inverted = True
 				# Invert after resize (avoids jaggies)
-				if inverted:
+				if inverted or color:
 					if not img:
 						img = bmp.ConvertToImage()
-					img.Invert()
 					alpha = wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNTEXT).alpha
 					if oname in ["applications-system", "color",
 								 "document-open", "document-save-as",
@@ -390,11 +394,18 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 						# Scale 85 to 255 and adjust alpha
 						factors = (3, 3, 3, alpha / 255.0)
 					else:
-						# Only adjust alpha
-						factors = (1, 1, 1, alpha / 255.0)
-				if color or factors:
-					if not img:
-						img = bmp.ConvertToImage()
+						if inverted:
+							img.Invert()
+						if alpha != 255:
+							# Only adjust alpha
+							factors = (1, 1, 1, alpha / 255.0)
+					if factors:
+						R, G, B = factors[:3]
+						if len(factors) > 3:
+							alpha = factors[3]
+						else:
+							alpha = 1.0
+						img = img.AdjustChannels(R, G, B, alpha)
 					if color:
 						# Hex format, RRGGBB or RRGGBBAA
 						R = int(color[0:2], 16) / 255.0
@@ -404,13 +415,7 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 							alpha = int(color[6:8], 16) / 255.0
 						else:
 							alpha = 1.0
-					else:
-						R, G, B = factors[:3]
-						if len(factors) > 3:
-							alpha = factors[3]
-						else:
-							alpha = 1.0
-					img = img.AdjustChannels(R, G, B, alpha)
+						img = img.AdjustChannels(R, G, B, alpha)
 				if img:
 					bmp = img.ConvertToBitmap()
 					if not bmp.IsOk():
@@ -716,7 +721,12 @@ valid_ranges = {
 	"app.port": [1, 65535],
 	"gamma": [0.000001, 10],
 	"trc": [0.000001, 10],
-	"calibration.ambient_viewcond_adjust.lux": [0, sys.maxint],
+	# Argyll dispcal uses 20% of ambient (in lux,
+	# fixed steradiant of 3.1415) as adapting
+	# luminance, but we assume it already *is*
+	# the adapting luminance. To correct for this,
+	# scale so that dispcal gets the correct value.
+	"calibration.ambient_viewcond_adjust.lux": [0.0, sys.maxint / 5.0],
 	"calibration.black_luminance": [0.000001, 10],
 	"calibration.black_output_offset": [0, 1],
 	"calibration.black_point_correction": [0, 1],
@@ -756,8 +766,8 @@ valid_values = {
 	# collink: xvYCC output encoding is not supported
 	"3dlut.encoding.output": filter(lambda v: v not in ("T", "x", "X"),
 									video_encodings),
-	"3dlut.format": ["3dl", "cube", "eeColor", "icc", "madVR", "mga", "png",
-					 "ReShade", "spi3d"],
+	"3dlut.format": ["3dl", "cube", "dcl", "eeColor", "icc", "madVR", "mga",
+					 "png", "ReShade", "spi3d"],
 	"3dlut.hdr_display": [0, 1],
 	"3dlut.image.layout": ["h", "v"],
 	"3dlut.image.order": ["rgb", "bgr"],
@@ -882,7 +892,7 @@ defaults = {
 	"drift_compensation.blacklevel": 0,
 	"drift_compensation.whitelevel": 0,
 	"calibration.ambient_viewcond_adjust": 0,
-	"calibration.ambient_viewcond_adjust.lux": 500.0,
+	"calibration.ambient_viewcond_adjust.lux": 32.0,
 	"calibration.autoload": 0,
 	"calibration.black_luminance": 0.000001,
 	"calibration.black_luminance.backup": 0.000001,
@@ -910,7 +920,7 @@ defaults = {
 	"colorimeter_correction.instrument.reference": None,
 	"colorimeter_correction.measurement_mode": "l",
 	"colorimeter_correction.measurement_mode.reference.adaptive": 1,
-	"colorimeter_correction.measurement_mode.reference.highres": 0,
+	"colorimeter_correction.measurement_mode.reference.highres": 1,
 	"colorimeter_correction.measurement_mode.reference.projector": 0,
 	"colorimeter_correction.measurement_mode.reference": "l",
 	"colorimeter_correction.observer": "1931_2",
@@ -923,9 +933,9 @@ defaults = {
 	# Note: worker.Worker.enumerate_displays_and_ports() overwrites copyright
 	"copyright": "No copyright. Created with %s %s and ArgyllCMS" % (appname, 
 																	 version),
-	"dimensions.measureframe": "0.5,0.5,1.5",
-	"dimensions.measureframe.unzoomed": "0.5,0.5,1.5",
-	"dimensions.measureframe.whitepoint.visual_editor": "0.5,0.5,1.5",
+	"dimensions.measureframe": "0.5,0.5,1.0",
+	"dimensions.measureframe.unzoomed": "0.5,0.5,1.0",
+	"dimensions.measureframe.whitepoint.visual_editor": "0.5,0.5,1.0",
 	"display.number": 1,
 	"display_lut.link": 1,
 	"display_lut.number": 1,
@@ -961,7 +971,7 @@ defaults = {
 	"measurement_mode": "l",
 	"measurement_mode.adaptive": 1,
 	"measurement_mode.backup": "l",
-	"measurement_mode.highres": 0,
+	"measurement_mode.highres": 1,
 	"measurement_mode.projector": 0,
 	"measurement_report.apply_black_offset": 0,
 	"measurement_report.apply_trc": 0,
@@ -1008,6 +1018,7 @@ defaults = {
 	"patterngenerator.quantize_bits": 0,
 	"patterngenerator.resolve": "CM",
 	"patterngenerator.resolve.port": 20002,
+	"patterngenerator.use_pattern_window": 0,
 	"patterngenerator.use_video_levels": 0,
 	"position.x": 50,
 	"position.y": 50,
@@ -1094,6 +1105,8 @@ defaults = {
 										   "spyder5elite.exe",
 										   "spyder5express.exe",
 										   "spyder5pro.exe",
+										   "spyderxelite.exe",
+										   "spyderxpro.exe",
 										   "dispcal.exe",
 										   "dispread.exe",
 										   "dispwin.exe",
@@ -1234,7 +1247,7 @@ def _init_testcharts():
 		testchart_defaults[key] = testchart_defaults["l"]
 
 
-def getcfg(name, fallback=True, raw=False):
+def getcfg(name, fallback=True, raw=False, cfg=cfg):
 	"""
 	Get and return an option value from the configuration.
 	
@@ -1361,7 +1374,7 @@ def getcfg(name, fallback=True, raw=False):
 	return value
 
 
-def hascfg(name, fallback=True):
+def hascfg(name, fallback=True, cfg=cfg):
 	"""
 	Check if an option name exists in the configuration.
 	
@@ -1389,7 +1402,7 @@ def get_current_profile(include_display_profile=False):
 	if path:
 		import ICCProfile as ICCP
 		try:
-			profile = ICCP.ICCProfile(path)
+			profile = ICCP.ICCProfile(path, use_cache=True)
 		except (IOError, ICCP.ICCProfileInvalidError), exception:
 			return
 		return profile
@@ -1457,7 +1470,7 @@ def get_standard_profiles(paths_only=False):
 						other_icc.append(os.path.join(dirpath, basename))
 		for path in ref_icc + other_icc:
 			try:
-				profile = ICCP.ICCProfile(path, load=False)
+				profile = ICCP.ICCProfile(path, load=False, use_cache=True)
 			except EnvironmentError:
 				pass
 			except Exception, exception:
@@ -1572,7 +1585,7 @@ def is_profile(filename=None, include_display_profile=False):
 		if os.path.exists(filename):
 			import ICCProfile as ICCP
 			try:
-				profile = ICCP.ICCProfile(filename)
+				profile = ICCP.ICCProfile(filename, use_cache=True)
 			except (IOError, ICCP.ICCProfileInvalidError):
 				pass
 			else:
@@ -1614,7 +1627,9 @@ def makecfgdir(which="user", worker=None):
 	return True
 
 
-def initcfg(module=None):
+cfginited = {}
+
+def initcfg(module=None, cfg=cfg, force_load=False):
 	"""
 	Initialize the configuration.
 	
@@ -1631,17 +1646,12 @@ def initcfg(module=None):
 	   not os.path.exists(os.path.join(confighome, cfgbasename + ".ini")):
 		# Set a few defaults which have None as possible value and thus cannot
 		# be set in the 'defaults' collection
-		setcfg("gamap_src_viewcond", "mt")
-		setcfg("gamap_out_viewcond", "mt")
+		setcfg("gamap_src_viewcond", "mt", cfg=cfg)
+		setcfg("gamap_out_viewcond", "mt", cfg=cfg)
 		# Set default preset
-		setcfg("calibration.file", defaults["calibration.file"])
+		setcfg("calibration.file", defaults["calibration.file"], cfg=cfg)
 	# Read cfg
-	cfgnames = []
-	if module not in ("3DLUT-maker", "VRML-to-X3D-converter",
-					  "scripting-client"):
-		# Never read base app cfg for 3D LUT maker, VRML converter and
-		# scripting client
-		cfgnames.append(appbasename)
+	cfgnames = [appbasename]
 	if module:
 		cfgnames.append(cfgbasename)
 	else:
@@ -1655,13 +1665,22 @@ def initcfg(module=None):
 		for cfgroot in cfgroots:
 			cfgfile = os.path.join(cfgroot, cfgname + ".ini")
 			if os.path.isfile(cfgfile):
-				cfgfiles.append(cfgfile)
+				try:
+					mtime = os.stat(cfgfile).st_mtime
+				except EnvironmentError, exception:
+					safe_print(u"Warning - os.stat('%s') failed: %s" % 
+							   tuple(safe_unicode(s) for s in (cfgfile,
+															   exception)))
+				if force_load or mtime != cfginited.get(cfgfile):
+					cfginited[cfgfile] = mtime
+					cfgfiles.append(cfgfile)
 				# Make user config take precedence
 				break
-	if len(cfgfiles) > 1 and (module != "apply-profiles" or
-							  sys.platform != "win32"):
+	if not cfgfiles:
+		return
+	if not module:
 		# Make most recent file take precedence
-		cfgfiles.sort(key=lambda cfgfile: os.stat(cfgfile).st_mtime)
+		cfgfiles.sort(key=lambda cfgfile: cfginited.get(cfgfile))
 	try:
 		cfg.read(cfgfiles)
 		# This won't raise an exception if the file does not exist, only
@@ -1678,6 +1697,10 @@ def initcfg(module=None):
 			for name, val in cfg.items(section):
 				if isinstance(val, list):
 					cfg.set(section, name, "\n".join(val))
+	finally:
+		if not module and not getcfg("calibration.ambient_viewcond_adjust"):
+			# Reset to default
+			setcfg("calibration.ambient_viewcond_adjust.lux", None, cfg=cfg)
 
 
 dpiset = False
@@ -1688,37 +1711,149 @@ def set_default_app_dpi():
 	global dpiset
 	if not dpiset and not getcfg("app.dpi", False):
 		# HighDPI support
-		dpiset = True
 		from wxaddons import wx
+		dpiset = True
 		if sys.platform in ("darwin", "win32"):
 			# Determine screen DPI
 			dpi = wx.ScreenDC().GetPPI()[0]
 		else:
-			if "gtk3" in wx.PlatformInfo:
-				from util_os import which
-				txt_scale = 1
-				if which("gsettings"):
-					import subprocess as sp
-					p = sp.Popen(["gsettings", "get", "org.gnome.desktop.interface",
-								  "text-scaling-factor"], stdin=sp.PIPE,
-								 stdout=sp.PIPE, stderr=sp.PIPE)
-					stdout, stderr = p.communicate()
-					if stdout:
-						try:
-							txt_scale = float(stdout)
-						except ValueError:
-							pass
-			else:
-				# Linux. Determine font scaling factor
-				font = wx.Font(256, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
-							   wx.FONTWEIGHT_NORMAL)
-				txt_scale = font.GetPixelSize()[0] / 256.0 or 1
-			dpi = int(round(get_default_dpi() * txt_scale))
+			# Linux
+			from util_os import which
+			txt_scale = None
+			# XDG_CURRENT_DESKTOP delimiter is colon (':')
+			desktop = os.getenv("XDG_CURRENT_DESKTOP", "").split(":")
+			if "gtk2" in wx.PlatformInfo:
+				txt_scale = get_hidpi_scaling_factor()
+			elif desktop[0] == "KDE":
+				pass
+				# Nothing to do
+			elif which("gsettings"):
+				import subprocess as sp
+				p = sp.Popen(["gsettings", "get", "org.gnome.desktop.interface",
+							  "text-scaling-factor"], stdin=sp.PIPE,
+							 stdout=sp.PIPE, stderr=sp.PIPE)
+				factor, stderr = p.communicate()
+				try:
+					txt_scale = float(factor)
+				except ValueError:
+					pass
+			dpi = get_default_dpi()
+			if txt_scale:
+				dpi = int(round(dpi * txt_scale))
 		defaults["app.dpi"] = dpi
 	dpiset = True
 
 
-def setcfg(name, value):
+def get_hidpi_scaling_factor():
+	if sys.platform in ("darwin", "win32"):
+		return 1.0  # Handled via app DPI
+	else:
+		# Linux
+		from util_os import which
+		if which("xrdb"):
+			import subprocess as sp
+			p = sp.Popen(["xrdb", "-query"], stdin=sp.PIPE,
+						 stdout=sp.PIPE, stderr=sp.PIPE)
+			# Format: 'Xft.dpi:        192'
+			stdout, stderr = p.communicate()
+			for line in stdout.splitlines():
+				if line.startswith("Xft.dpi:"):
+					split = line.split()
+					dpi = split[-1]
+					try:
+						return float(dpi) / get_default_dpi()
+					except ValueError:
+						pass
+		factor = None
+		# XDG_CURRENT_DESKTOP delimiter is colon (':')
+		desktop = os.getenv("XDG_CURRENT_DESKTOP", "").split(":")
+		if desktop[0] == "KDE":
+			# Two env-vars exist: QT_SCALE_FACTOR and
+			# QT_SCREEN_SCALE_FACTORS.
+			# According to documentation[1], the latter is
+			# 'mainly useful for debugging' - that's not how it is
+			# used by KDE though. Changing display scaling via KDE
+			# settings GUI only sets QT_SCREEN_SCALE_FACTORS.
+			# We are thus currently ignoring QT_SCALE_FACTOR.
+			# [1] https://doc.qt.io/qt-5/highdpi.html
+			# QT_SCREEN_SCALE_FACTORS delimiter is semicolon (';')
+			# Format: Mapping of XrandR display names to scale factor
+			# e.g. 'VGA-1=1.5;VGA-2=2.0;'
+			# or just list of scale factors
+			# e.g. '1.5;2.0;'
+			screen_scale_factors = os.getenv("QT_SCREEN_SCALE_FACTORS", "").split(";")
+			if screen_scale_factors:
+				from wxaddons import wx
+				match = False
+				app = wx.GetApp()
+				if app:
+					import RealDisplaySizeMM as RDSMM
+					if not RDSMM._displays:
+						RDSMM.enumerate_displays()
+					top = app.TopWindow
+					if top:
+						tmp = False
+					else:
+						# Create temp frame if no topwindow
+						top = wx.Frame(None)
+						# Move to main window location (and thus screen)
+						x, y = (getcfg("position.x", False),
+								getcfg("position.y", False))
+						if not None in (x, y):
+							top.SetSaneGeometry(x, y)
+						tmp = True
+					# Get wx display
+					wx_display = top.GetDisplay()
+					if tmp:
+						# No longer need our temp frame
+						top.Destroy()
+					# Search for matching display based on geometry
+					pos = wx_display.Geometry[:2]
+					size = wx_display.Geometry[2:]
+					for item in screen_scale_factors:
+						if not item:
+							break
+						if "=" in item:
+							name, factor = item.split("=", 1)
+						else:
+							name, factor = None, item
+						for display in RDSMM._displays:
+							if (display.get("pos") != pos or
+								display.get("size") != size):
+								# No match
+								continue
+							if (name and
+								display.get("xrandr_name") != name):
+								# No match
+								continue
+							# Match found
+							match = True
+							break
+						if match:
+							break
+				if not match:
+					# Use first one
+					factor = screen_scale_factors[0].split("=")[-1]
+		if not factor and which("gsettings"):
+			# GNOME
+			import subprocess as sp
+			p = sp.Popen(["gsettings", "get", "org.gnome.desktop.interface",
+						  "scaling-factor"], stdin=sp.PIPE,
+						 stdout=sp.PIPE, stderr=sp.PIPE)
+			# Format: 'unint32 1'
+			stdout, stderr = p.communicate()
+			split = stdout.split()
+			if split:
+				factor = split[-1]
+		if factor is not None:
+			try:
+				factor = float(factor)
+			except ValueError:
+				factor = None
+		return factor
+
+
+def setcfg(name, value, cfg=cfg):
 	""" Set an option value in the configuration. """
 	if value is None:
 		cfg.remove_option(ConfigParser.DEFAULTSECT, name)
@@ -1731,7 +1866,7 @@ def setcfg(name, value):
 		cfg.set(ConfigParser.DEFAULTSECT, name, unicode(value).encode("UTF-8"))
 
 
-def writecfg(which="user", worker=None, module=None, options=()):
+def writecfg(which="user", worker=None, module=None, options=(), cfg=cfg):
 	"""
 	Write configuration file.
 	
